@@ -460,6 +460,17 @@ parser_c::frames_available()
   return m_frames.size();
 }
 
+parser_c::multiplex_type_e
+parser_c::get_multiplex_type()
+  const {
+  return m_multiplex_type;
+}
+
+void
+parser_c::set_multiplex_type(multiplex_type_e multiplex_type) {
+  m_multiplex_type = multiplex_type;
+}
+
 frame_c
 parser_c::get_frame() {
   auto frame = m_frames.front();
@@ -529,6 +540,9 @@ parser_c::decode_adts_header(unsigned char const *buffer,
     frame.m_header.data_byte_size   = frame.m_header.bytes - frame.m_header.header_byte_size;
     frame.m_header.is_valid         = true;
 
+    if (frame.m_header.bytes <= frame.m_header.header_byte_size)
+      return { failure, 1 };
+
     if (m_copy_data) {
       frame.m_data = memory_c::alloc(frame.m_header.data_byte_size);
       bc.get_bytes(frame.m_data->get_buffer(), frame.m_header.data_byte_size);
@@ -702,6 +716,14 @@ parser_c::parse() {
     m_buffer.remove(position);
 }
 
+std::string
+parser_c::get_multiplex_type_name(multiplex_type_e multiplex_type) {
+  return multiplex_type == adts_multiplex      ? "ADTS"
+       : multiplex_type == adts_multiplex      ? "ADIF"
+       : multiplex_type == loas_latm_multiplex ? "LOAS/LATM"
+       :                                         "unknown";
+}
+
 int
 parser_c::find_consecutive_frames(unsigned char const *buffer,
                                   size_t buffer_size,
