@@ -57,7 +57,11 @@ char const * const playlist_duration               = "playlist_duration";
 char const * const playlist_file                   = "playlist_file";                   // container uinstr
 char const * const playlist_size                   = "playlist_size";                   // container unsigned-integer
 char const * const previous_segment_uid            = "previous_segment_uid";            // container ascii-string format:^[0-9A-F]{32}$
+char const * const program_number                  = "program_number";                  // track unsigned-integer, container unsigned-integer
+char const * const programs                        = "programs";                        // container array
 char const * const segment_uid                     = "segment_uid";                     // container ascii-string format:^[0-9A-F]{32}$
+char const * const service_name                    = "service_name";                    // container unicode-string
+char const * const service_provider                = "service_provider";                // container unicode-string
 char const * const stereo_mode                     = "stereo_mode";                     // track unsigned-integer
 char const * const stream_id                       = "stream_id";                       // track unsigned-integer
 char const * const sub_stream_id                   = "sub_stream_id";                   // track unsigned-integer
@@ -65,7 +69,6 @@ char const * const teletext_page                   = "teletext_page";
 char const * const text_subtitles                  = "text_subtitles";                  // track boolean
 char const * const title                           = "title";                           // container unicoode-string
 char const * const track_name                      = "track_name";                      // track unicoode-string
-char const * const ts_pid                          = "ts_pid";                          // track unsigned-integer
 char const * const uid                             = "uid";                             // track attachments unsigned-integer
 char const * const writing_application             = "writing_application";             // container unicode-string
 
