@@ -47,10 +47,36 @@ mpeg_ps_reader_c::probe_file(mm_io_c *in,
     if (in->read(buf, 4) != 4)
       return 0;
 
-    if (get_uint32_be(buf) != MPEGVIDEO_PACKET_START_CODE)
+    if (get_uint32_be(buf) == MPEGVIDEO_PACKET_START_CODE)
+      return 1;
+
+    in->setFilePointer(0, seek_beginning);
+
+    auto mem      = memory_c::alloc(32 * 1024);
+    auto num_read = in->read(mem, mem->get_size());
+
+    if (num_read < 4)
       return 0;
 
-    return 1;
+    auto base                           = mem->get_buffer();
+    auto code                           = get_uint32_be(base);
+    auto offset                         = 2u;
+    auto system_header_start_code_found = false;
+    auto packet_start_code_found        = false;
+
+    while(   ((offset + 4) < num_read)
+          && (!system_header_start_code_found || !packet_start_code_found)) {
+      ++offset;
+      code = (code << 8) | base[offset];
+
+      if (code == MPEGVIDEO_SYSTEM_HEADER_START_CODE)
+        system_header_start_code_found = true;
+
+      else if (code == MPEGVIDEO_PACKET_START_CODE)
+        packet_start_code_found = true;
+    }
+
+    return system_header_start_code_found && packet_start_code_found;
 
   } catch (...) {
     return 0;
@@ -63,6 +89,9 @@ mpeg_ps_reader_c::mpeg_ps_reader_c(const track_info_c &ti,
   , file_done(false)
   , m_probe_range{}
   , m_debug_timecodes{"mpeg_ps|mpeg_ps_timecodes"}
+  , m_debug_headers{  "mpeg_ps|mpeg_ps_headers"}
+  , m_debug_packets{  "mpeg_ps|mpeg_ps_packets"}
+  , m_debug_resync{   "mpeg_ps|mpeg_ps_resync"}
 {
 }
 
@@ -88,7 +117,7 @@ mpeg_ps_reader_c::read_headers() {
 
       switch (header) {
         case MPEGVIDEO_PACKET_START_CODE:
-          mxverb(3, boost::format("mpeg_ps: packet start at %1%\n") % (m_in->getFilePointer() - 4));
+          mxdebug_if(m_debug_headers, boost::format("mpeg_ps: packet start at %1%\n") % (m_in->getFilePointer() - 4));
 
           if (-1 == version) {
             byte = m_in->read_uint8();
@@ -109,7 +138,7 @@ mpeg_ps_reader_c::read_headers() {
           break;
 
         case MPEGVIDEO_SYSTEM_HEADER_START_CODE:
-          mxverb(3, boost::format("mpeg_ps: system header start code at %1%\n") % (m_in->getFilePointer() - 4));
+          mxdebug_if(m_debug_headers, boost::format("mpeg_ps: system header start code at %1%\n") % (m_in->getFilePointer() - 4));
 
           m_in->skip(2 * 4);   // system header
           byte = m_in->read_uint8();
@@ -132,7 +161,7 @@ mpeg_ps_reader_c::read_headers() {
 
         default:
           if (!mpeg_is_start_code(header)) {
-            mxverb(3, boost::format("mpeg_ps: unknown header 0x%|1$08x| at %2%\n") % header % (m_in->getFilePointer() - 4));
+            mxdebug_if(m_debug_headers, boost::format("mpeg_ps: unknown header 0x%|1$08x| at %2%\n") % header % (m_in->getFilePointer() - 4));
             done = !resync_stream(header);
             break;
           }
@@ -143,7 +172,7 @@ mpeg_ps_reader_c::read_headers() {
           m_in->restore_pos();
           pes_packet_length = m_in->read_uint16_be();
 
-          mxverb(3, boost::format("mpeg_ps: id 0x%|1$02x| len %2% at %3%\n") % static_cast<unsigned int>(stream_id) % pes_packet_length % (m_in->getFilePointer() - 4 - 2));
+          mxdebug_if(m_debug_headers, boost::format("mpeg_ps: id 0x%|1$02x| len %2% at %3%\n") % static_cast<unsigned int>(stream_id) % pes_packet_length % (m_in->getFilePointer() - 4 - 2));
 
           m_in->skip(pes_packet_length);
 
@@ -190,10 +219,14 @@ mpeg_ps_reader_c::sort_tracks() {
   for (i = 0; tracks.size() > i; ++i)
     id2idx[tracks[i]->id.idx()] = i;
 
-  mxverb(2, "mpeg_ps: Supported streams, sorted by ID: ");
+  if (!m_debug_headers)
+    return;
+
+  auto info = std::string{"mpeg_ps: Supported streams, sorted by ID: "};
   for (i = 0; tracks.size() > i; ++i)
-    mxverb(2, boost::format("0x%|1$02x|(0x%|2$02x|) ") % tracks[i]->id.id % tracks[i]->id.sub_id);
-  mxverb(2, "\n");
+    info += (boost::format("0x%|1$02x|(0x%|2$02x|) ") % tracks[i]->id.id % tracks[i]->id.sub_id).str();
+
+  mxdebug(info + "\n");
 }
 
 void
@@ -313,12 +346,12 @@ mpeg_ps_reader_c::parse_packet(mpeg_ps_id_t id,
       m_in->setFilePointer(pos + packet.m_length);
 
     else {
-      mxverb(2, boost::format("mpeg_ps: [begin] padding stream length incorrect at %1%, find next header...\n") % (pos - 6));
+      mxdebug_if(m_debug_packets, boost::format("mpeg_ps: [begin] padding stream length incorrect at %1%, find next header...\n") % (pos - 6));
       m_in->setFilePointer(pos);
       header = 0xffffffff;
       if (resync_stream(header)) {
         packet.m_full_length = m_in->getFilePointer() - pos - 4;
-        mxverb(2, boost::format("mpeg_ps: [end] padding stream length adjusted from %1% to %2%\n") % packet.m_length % packet.m_full_length);
+        mxdebug_if(m_debug_packets, boost::format("mpeg_ps: [end] padding stream length adjusted from %1% to %2%\n") % packet.m_length % packet.m_full_length);
         m_in->setFilePointer(pos + packet.m_full_length);
       }
     }
@@ -650,7 +683,7 @@ mpeg_ps_reader_c::new_stream_v_mpeg_1_2(mpeg_ps_id_t id,
   }
 
   if ((MPV_PARSER_STATE_FRAME != state) || !found_i_frame || !m2v_parser->GetMPEGVersion() || !seq_hdr.width || !seq_hdr.height) {
-    mxverb(3, boost::format("MPEG PS: blacklisting id 0x%|1$02x|(%|2$02x|) for supposed type MPEG1/2\n") % id.id % id.sub_id);
+    mxdebug_if(m_debug_headers, boost::format("MPEG PS: blacklisting id 0x%|1$02x|(%|2$02x|) for supposed type MPEG1/2\n") % id.id % id.sub_id);
     blacklisted_ids[id.idx()] = true;
     throw false;
   }
@@ -773,21 +806,42 @@ mpeg_ps_reader_c::new_stream_a_mpeg(mpeg_ps_id_t,
 }
 
 void
-mpeg_ps_reader_c::new_stream_a_ac3(mpeg_ps_id_t,
+mpeg_ps_reader_c::new_stream_a_ac3(mpeg_ps_id_t id,
                                    unsigned char *buf,
                                    unsigned int length,
                                    mpeg_ps_track_ptr &track) {
-  ac3::frame_c header;
-  if (-1 == header.find_in(buf, length))
-    throw false;
+  mxdebug_if(m_debug_headers, boost::format("new_stream_a_ac3 for ID %1% buf len %2%\n") % id % length);
 
-  mxverb(2,
-         boost::format("first ac3 header bsid %1% channels %2% sample_rate %3% bytes %4% samples %5%\n")
-         % header.m_bs_id % header.m_channels % header.m_sample_rate % header.m_bytes % header.m_samples);
+  byte_buffer_c buffer;
 
-  track->a_channels    = header.m_channels;
-  track->a_sample_rate = header.m_sample_rate;
-  track->a_bsid        = header.m_bs_id;
+  buffer.add(buf, length);
+
+  while (m_probe_range >= m_in->getFilePointer()) {
+    ac3::frame_c header;
+
+    if (-1 != header.find_in(buffer.get_buffer(), buffer.get_size())) {
+      mxdebug_if(m_debug_headers,
+                 boost::format("new_stream_a_ac3 first AC-3 header bsid %1% channels %2% sample_rate %3% bytes %4% samples %5%\n")
+                 % header.m_bs_id % header.m_channels % header.m_sample_rate % header.m_bytes % header.m_samples);
+
+      track->a_channels    = header.m_channels;
+      track->a_sample_rate = header.m_sample_rate;
+      track->a_bsid        = header.m_bs_id;
+
+      return;
+    }
+
+    if (!find_next_packet_for_id(id, m_probe_range))
+      throw false;
+
+    auto packet = parse_packet(id);
+    if (!packet || (id.sub_id != packet.m_id.sub_id))
+      continue;
+
+    buffer.add(packet.m_buffer->get_buffer(), packet.m_length);
+  }
+
+  throw false;
 }
 
 void
@@ -831,9 +885,9 @@ mpeg_ps_reader_c::new_stream_a_truehd(mpeg_ps_id_t id,
       if (truehd_frame_t::sync != frame->m_type)
         continue;
 
-      mxverb(2,
-             boost::format("first TrueHD header channels %1% sampling_rate %2% samples_per_frame %3%\n")
-             % frame->m_channels % frame->m_sampling_rate % frame->m_samples_per_frame);
+      mxdebug_if(m_debug_headers,
+                 boost::format("first TrueHD header channels %1% sampling_rate %2% samples_per_frame %3%\n")
+                 % frame->m_channels % frame->m_sampling_rate % frame->m_samples_per_frame);
 
       track->codec         = frame->codec();
       track->a_channels    = frame->m_channels;
@@ -901,7 +955,7 @@ mpeg_ps_reader_c::new_stream_a_pcm(mpeg_ps_id_t,
 
 void
 mpeg_ps_reader_c::found_new_stream(mpeg_ps_id_t id) {
-  mxverb(2, boost::format("MPEG PS: new stream id 0x%|1$02x|\n") % id.id);
+  mxdebug_if(m_debug_headers, boost::format("MPEG PS: new stream id 0x%|1$02x|\n") % id.id);
 
   if (((0xc0 > id.id) || (0xef < id.id)) && (0xbd != id.id) && (0xfd != id.id))
     return;
@@ -914,7 +968,7 @@ mpeg_ps_reader_c::found_new_stream(mpeg_ps_id_t id) {
     id = packet.m_id;
 
     if (0xbd == id.id) {        // DVD audio substream
-      mxverb(2, boost::format("MPEG PS:   audio substream id 0x%|1$02x|\n") % id.sub_id);
+      mxdebug_if(m_debug_headers, boost::format("MPEG PS:   audio substream id 0x%|1$02x|\n") % id.sub_id);
       if (0 == id.sub_id)
         return;
     }
@@ -1146,7 +1200,7 @@ mpeg_ps_reader_c::find_next_packet_for_id(mpeg_ps_id_t id,
 
 bool
 mpeg_ps_reader_c::resync_stream(uint32_t &header) {
-  mxverb(2, boost::format("MPEG PS: synchronisation lost at %1%; looking for start code\n") % m_in->getFilePointer());
+  mxdebug_if(m_debug_resync, boost::format("MPEG PS: synchronisation lost at %1%; looking for start code\n") % m_in->getFilePointer());
 
   try {
     while (1) {
@@ -1156,12 +1210,12 @@ mpeg_ps_reader_c::resync_stream(uint32_t &header) {
         break;
     }
 
-    mxverb(2, boost::format("resync succeeded at %1%, header 0x%|2$08x|\n") % (m_in->getFilePointer() - 4) % header);
+    mxdebug_if(m_debug_resync, boost::format("resync succeeded at %1%, header 0x%|2$08x|\n") % (m_in->getFilePointer() - 4) % header);
 
     return true;
 
   } catch (...) {
-    mxverb(2, "resync failed: exception caught\n");
+    mxdebug_if(m_debug_resync, "resync failed: exception caught\n");
     return false;
   }
 }
@@ -1270,7 +1324,7 @@ mpeg_ps_reader_c::read(generic_packetizer_c *requested_ptzr,
       if (!packet) {
         if (    (0xbe != new_id.id)       // padding stream
              && (0xbf != new_id.id))      // private 2 stream (navigation data)
-          mxverb(2, boost::format("mpeg_ps: parse_packet failed at %1%, skipping %2%\n") % packet_pos % packet.m_full_length);
+          mxdebug_if(m_debug_packets, boost::format("mpeg_ps: parse_packet failed at %1%, skipping %2%\n") % packet_pos % packet.m_full_length);
         m_in->setFilePointer(packet_pos + 4 + 2 + packet.m_full_length);
         continue;
       }
@@ -1280,7 +1334,7 @@ mpeg_ps_reader_c::read(generic_packetizer_c *requested_ptzr,
         continue;
       }
 
-      mxverb(3, boost::format("mpeg_ps: packet at %1%: %2%\n") % packet_pos % packet);
+      mxdebug_if(m_debug_packets, boost::format("mpeg_ps: packet at %1%: %2%\n") % packet_pos % packet);
 
       auto track = tracks[id2idx[new_id.idx()]];
 
@@ -1313,7 +1367,7 @@ mpeg_ps_reader_c::read(generic_packetizer_c *requested_ptzr,
         track->assert_buffer_size(packet.m_length);
 
         if (m_in->read(&track->buffer[track->buffer_usage], packet.m_length) != packet.m_length) {
-          mxverb(2, "mpeg_ps: file_done: m_in->read\n");
+          mxdebug_if(m_debug_packets, "mpeg_ps: file_done: m_in->read\n");
           return finish();
         }
 
@@ -1326,7 +1380,7 @@ mpeg_ps_reader_c::read(generic_packetizer_c *requested_ptzr,
         auto buf = memory_c::alloc(packet.m_length);
 
         if (m_in->read(buf, packet.m_length) != packet.m_length) {
-          mxverb(2, "mpeg_ps: file_done: m_in->read\n");
+          mxdebug_if(m_debug_packets, "mpeg_ps: file_done: m_in->read\n");
           return finish();
         }
 
@@ -1335,10 +1389,10 @@ mpeg_ps_reader_c::read(generic_packetizer_c *requested_ptzr,
 
       return FILE_STATUS_MOREDATA;
     }
-    mxverb(2, "mpeg_ps: file_done: !find_next_packet\n");
+    mxdebug_if(m_debug_packets, "mpeg_ps: file_done: !find_next_packet\n");
 
   } catch(...) {
-    mxverb(2, "mpeg_ps: file_done: exception\n");
+    mxdebug_if(m_debug_packets, "mpeg_ps: file_done: exception\n");
   }
 
   return finish();
@@ -1378,8 +1432,8 @@ mpeg_ps_reader_c::identify() {
       info.add(mtx::id::packetizer, mtx::id::mpeg4_p10_es_video);
 
     info.set(mtx::id::number,        (static_cast<uint64_t>(track->id.sub_id) << 32) | static_cast<uint64_t>(track->id.id));
-    info.add(mtx::id::stream_id,     boost::format("%|1$02x|") % track->id.id);
-    info.add(mtx::id::sub_stream_id, boost::format("%|1$02x|") % track->id.sub_id);
+    info.add(mtx::id::stream_id,     track->id.id);
+    info.add(mtx::id::sub_stream_id, track->id.sub_id);
 
     if ((0 != track->v_dwidth) && (0 != track->v_dheight))
       info.add(mtx::id::display_dimensions, boost::format("%1%x%2%") % track->v_dwidth % track->v_dheight);
