@@ -70,6 +70,7 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *reader,
   , m_htrack_min_cache{}
   , m_htrack_max_cache{-1}
   , m_htrack_default_duration{-1}
+  , m_htrack_default_duration_indicates_fields{}
   , m_default_duration_forced{true}
   , m_default_track_warning_printed{}
   , m_huid{}
@@ -290,11 +291,15 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *reader,
     set_video_stereo_mode(m_ti.m_stereo_mode_list[m_ti.m_id], OPTION_SOURCE_COMMAND_LINE);
 
   // Let's see if the user has specified a default duration for this track.
-  if (mtx::includes(m_ti.m_default_durations, m_ti.m_id))
-    m_htrack_default_duration = m_ti.m_default_durations[m_ti.m_id];
-  else if (mtx::includes(m_ti.m_default_durations, -1))
-    m_htrack_default_duration = m_ti.m_default_durations[-1];
-  else
+  if (mtx::includes(m_ti.m_default_durations, m_ti.m_id)) {
+    m_htrack_default_duration                  = m_ti.m_default_durations[m_ti.m_id].first;
+    m_htrack_default_duration_indicates_fields = m_ti.m_default_durations[m_ti.m_id].second;
+
+  } else if (mtx::includes(m_ti.m_default_durations, -1)) {
+    m_htrack_default_duration                  = m_ti.m_default_durations[-1].first;
+    m_htrack_default_duration_indicates_fields = m_ti.m_default_durations[-1].second;
+
+  } else
     m_default_duration_forced = false;
 
   // Let's see if the user has set a max_block_add_id
@@ -324,8 +329,10 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *reader,
   // If no external timecode file but a default duration has been
   // given then create a simple timecode factory that generates the
   // timecodes for the given FPS.
-  if (!m_timestamp_factory && (-1 != m_htrack_default_duration))
-    m_timestamp_factory = timestamp_factory_c::create_fps_factory(m_htrack_default_duration, m_ti.m_tcsync);
+  if (!m_timestamp_factory && (-1 != m_htrack_default_duration)) {
+    auto divisor        = m_htrack_default_duration_indicates_fields ? 2 : 1;
+    m_timestamp_factory = timestamp_factory_c::create_fps_factory(m_htrack_default_duration / divisor, m_ti.m_tcsync);
+  }
 }
 
 generic_packetizer_c::~generic_packetizer_c() {
@@ -444,8 +451,9 @@ generic_packetizer_c::set_track_max_cache(int max_cache) {
 }
 
 void
-generic_packetizer_c::set_track_default_duration(int64_t def_dur) {
-  if (m_default_duration_forced)
+generic_packetizer_c::set_track_default_duration(int64_t def_dur,
+                                                 bool force) {
+  if (!force && m_default_duration_forced)
     return;
 
   m_htrack_default_duration = (int64_t)(def_dur * m_ti.m_tcsync.numerator / m_ti.m_tcsync.denominator);
@@ -1168,6 +1176,29 @@ generic_packetizer_c::fix_headers() {
   m_track_entry->SetGlobalTimecodeScale((int64_t)g_timecode_scale);
 }
 
+void
+generic_packetizer_c::compress_packet(packet_t &packet) {
+  if (!m_compressor) {
+    return;
+  }
+
+  try {
+    packet.data = m_compressor->compress(packet.data);
+    size_t i;
+    for (i = 0; packet.data_adds.size() > i; ++i)
+      packet.data_adds[i] = m_compressor->compress(packet.data_adds[i]);
+
+  } catch (mtx::compression_x &e) {
+    mxerror_tid(m_ti.m_fname, m_ti.m_id, boost::format(Y("Compression failed: %1%\n")) % e.error());
+  }
+}
+
+void
+generic_packetizer_c::account_enqueued_bytes(packet_t &packet,
+                                             int64_t factor) {
+  m_enqueued_bytes += packet.calculate_uncompressed_size() * factor;
+}
+
 void
 generic_packetizer_c::add_packet(packet_cptr pack) {
   if ((0 == m_num_packets) && m_ti.m_reset_timecodes)
@@ -1192,6 +1223,8 @@ generic_packetizer_c::add_packet(packet_cptr pack) {
   if ((0 > pack->bref) && (0 <= pack->fref))
     std::swap(pack->bref, pack->fref);
 
+  account_enqueued_bytes(*pack, +1);
+
   if (1 != m_connected_to)
     add_packet2(pack);
   else
@@ -1270,22 +1303,7 @@ generic_packetizer_c::add_packet2(packet_cptr pack) {
 
   after_packet_timestamped(*pack);
 
-  if (!m_compressor) {
-    m_enqueued_bytes += pack->data->get_size();
-    return;
-  }
-
-  try {
-    pack->data = m_compressor->compress(pack->data);
-    size_t i;
-    for (i = 0; pack->data_adds.size() > i; ++i)
-      pack->data_adds[i] = m_compressor->compress(pack->data_adds[i]);
-
-    m_enqueued_bytes += pack->data->get_size();
-
-  } catch (mtx::compression_x &e) {
-    mxerror_tid(m_ti.m_fname, m_ti.m_id, boost::format(Y("Compression failed: %1%\n")) % e.error());
-  }
+  compress_packet(*pack);
 }
 
 void
@@ -1305,7 +1323,7 @@ generic_packetizer_c::get_packet() {
 
   pack->output_order_timecode = timestamp_c::ns(pack->assigned_timecode - std::max(m_codec_delay.to_ns(0), m_seek_pre_roll.to_ns(0)));
 
-  m_enqueued_bytes -= pack->data->get_size();
+  account_enqueued_bytes(*pack, -1);
 
   --m_next_packet_wo_assigned_timecode;
   if (0 > m_next_packet_wo_assigned_timecode)
