@@ -109,6 +109,7 @@ bool g_no_lacing                            = false;
 bool g_no_linking                           = true;
 bool g_use_durations                        = false;
 bool g_no_track_statistics_tags             = false;
+bool g_write_date                           = true;
 
 double g_timecode_scale                     = TIMECODE_SCALE;
 timecode_scale_mode_e g_timecode_scale_mode = TIMECODE_SCALE_MODE_NORMAL;
@@ -573,7 +574,11 @@ render_headers(mm_io_c *out) {
 
     GetChild<KaxMuxingApp >(*s_kax_infos).SetValueUTF8(s_muxing_app);
     GetChild<KaxWritingApp>(*s_kax_infos).SetValueUTF8(s_writing_app);
-    GetChild<KaxDateUTC   >(*s_kax_infos).SetEpochDate(s_writing_date.is_not_a_date_time() ? 0 : mtx::date_time::to_time_t(s_writing_date));
+
+    if (g_write_date)
+      GetChild<KaxDateUTC>(*s_kax_infos).SetEpochDate(s_writing_date.is_not_a_date_time() ? 0 : mtx::date_time::to_time_t(s_writing_date));
+    else
+      DeleteChildren<KaxDateUTC>(*s_kax_infos);
 
     if (!g_segment_title.empty())
       GetChild<KaxTitle>(*s_kax_infos).SetValueUTF8(g_segment_title.c_str());
@@ -725,6 +730,9 @@ adjust_cue_and_seekhead_positions(uint64_t data_start_pos,
 static void
 relocate_written_data(uint64_t data_start_pos,
                       uint64_t delta) {
+  if (g_cluster_helper->discarding())
+    return;
+
   auto rel_pos_from_end = s_out->get_size() - s_out->getFilePointer();
   auto const block_size = 1024llu * 1024;
   auto to_relocate      = s_out->get_size() - data_start_pos;
