@@ -23,8 +23,17 @@ namespace Jobs {
 class Job;
 using JobPtr = std::shared_ptr<Job>;
 
+class JobPrivate;
 class Job: public QObject {
   Q_OBJECT;
+
+protected:
+  Q_DECLARE_PRIVATE(Job);
+
+  QScopedPointer<JobPrivate> const d_ptr;
+
+  explicit Job(JobPrivate &d);
+
   Q_ENUMS(Status);
 
 public:
@@ -45,22 +54,6 @@ public:
     ErrorLine,
   };
 
-private:
-  static uint64_t ms_next_id;
-
-protected:
-  QUuid m_uuid;
-  uint64_t m_id;
-  Status m_status;
-  QString m_description;
-  QStringList m_output, m_warnings, m_errors, m_fullOutput;
-  unsigned int m_progress, m_exitCode;
-  int m_warningsAcknowledged, m_errorsAcknowledged;
-  QDateTime m_dateAdded, m_dateStarted, m_dateFinished;
-  bool m_quitAfterFinished, m_modified;
-
-  QMutex m_mutex;
-
 public:
   Job(Status status = PendingManual);
   virtual ~Job();
@@ -110,11 +103,13 @@ public:
   int numUnacknowledgedWarnings() const;
   int numUnacknowledgedErrors() const;
 
+  virtual void runProgramSetupVariables(ProgramRunner::VariableMap &variables) const;
+
 protected:
   virtual void saveJobInternal(Util::ConfigFile &settings) const = 0;
   virtual void loadJobBasis(Util::ConfigFile &settings);
   virtual void runProgramsAfterCompletion();
-  virtual void runProgramSetupVariables(ProgramRunner::VariableMap &variables);
+  void setupJobConnections();
 
 public slots:
   virtual void setStatus(Job::Status status);
