@@ -467,7 +467,7 @@ Model::updateProgress() {
     }
 
   auto progress      = numRunning ? runningProgress / numRunning : 0u;
-  auto totalProgress = (m_queueNumDone * 100 + runningProgress) / (m_toBeProcessed.count() + m_queueNumDone);
+  auto totalProgress = m_toBeProcessed.count() != 0 ? (m_queueNumDone * 100 + runningProgress) / m_toBeProcessed.count() : 0u;
 
   emit progressChanged(progress, totalProgress);
 }
