@@ -18,21 +18,25 @@
 #include "mkvtoolnix-gui/main_window/prefs_run_program_widget.h"
 #include "mkvtoolnix-gui/merge/additional_command_line_options_dialog.h"
 #include "mkvtoolnix-gui/util/file_dialog.h"
+#include "mkvtoolnix-gui/util/message_box.h"
+#include "mkvtoolnix-gui/util/model.h"
 #include "mkvtoolnix-gui/util/side_by_side_multi_select.h"
 #include "mkvtoolnix-gui/util/widget.h"
 
 namespace mtx { namespace gui {
 
-PreferencesDialog::PreferencesDialog(QWidget *parent)
+PreferencesDialog::PreferencesDialog(QWidget *parent,
+                                     Page pageToShow)
   : QDialog{parent, Qt::Dialog | Qt::WindowMaximizeButtonHint | Qt::WindowCloseButtonHint}
   , ui{new Ui::PreferencesDialog}
   , m_cfg(Util::Settings::get())
   , m_previousUiLocale{m_cfg.m_uiLocale}
   , m_previousProbeRangePercentage{m_cfg.m_probeRangePercentage}
+  , m_ignoreNextCurrentChange{}
 {
   ui->setupUi(this);
 
-  setupPageSelector();
+  setupPageSelector(pageToShow);
 
   ui->tbOftenUsedXYZ->setTabPosition(m_cfg.m_tabPosition);
 
@@ -87,8 +91,9 @@ PreferencesDialog::PreferencesDialog(QWidget *parent)
   setupTrackPropertiesLayout();
 
   // Chapter editor page
-  ui->leCENameTemplate->setText(m_cfg.m_chapterNameTemplate);
   ui->cbCEDropLastFromBlurayPlaylist->setChecked(m_cfg.m_dropLastChapterFromBlurayPlaylist);
+  ui->cbCETextFileCharacterSet->setAdditionalItems(m_cfg.m_ceTextFileCharacterSet).setup(true, QY("Always ask the user")).setCurrentByData(m_cfg.m_ceTextFileCharacterSet);
+  ui->leCENameTemplate->setText(m_cfg.m_chapterNameTemplate);
   ui->cbCEDefaultLanguage->setAdditionalItems(m_cfg.m_defaultChapterLanguage).setup().setCurrentByData(m_cfg.m_defaultChapterLanguage);
   ui->cbCEDefaultCountry->setAdditionalItems(m_cfg.m_defaultChapterCountry).setup(true, QY("– No selection by default –")).setCurrentByData(m_cfg.m_defaultChapterCountry);
 
@@ -124,6 +129,11 @@ PreferencesDialog::pageSelectionChanged(QModelIndex const &current) {
   if (!current.isValid())
     return;
 
+  if (m_ignoreNextCurrentChange) {
+    m_ignoreNextCurrentChange = false;
+    return;
+  }
+
   auto page = qobject_cast<QStandardItemModel *>(ui->pageSelector->model())
     ->itemFromIndex(current.sibling(current.row(), 0))
     ->data()
@@ -131,17 +141,30 @@ PreferencesDialog::pageSelectionChanged(QModelIndex const &current) {
   ui->pages->setCurrentIndex(page);
 }
 
+QModelIndex
+PreferencesDialog::modelIndexForPage(int page) {
+  auto &model  = *qobject_cast<QStandardItemModel *>(ui->pageSelector->model());
+  auto pageIdx = Util::findIndex(model, [page, &model](QModelIndex const &idxToTest) {
+    return model.itemFromIndex(idxToTest)->data().value<int>() == page;
+  });
+
+  return pageIdx;
+}
+
 void
-PreferencesDialog::setupPageSelector() {
+PreferencesDialog::setupPageSelector(Page pageToShow) {
   m_cfg.handleSplitterSizes(ui->pagesSplitter);
 
-  auto model = new QStandardItemModel{};
+  auto pageIndex = 0;
+  auto model     = new QStandardItemModel{};
   ui->pageSelector->setModel(model);
 
-  auto addItem = [model](int page, QStandardItem *parent, QString const &text, QString const &icon = QString{}) -> QStandardItem * {
+  auto addItem = [this, model, &pageIndex](Page pageType, QStandardItem *parent, QString const &text, QString const &icon = QString{}) -> QStandardItem * {
     auto item = new QStandardItem{text};
 
-    item->setData(page);
+    item->setData(pageIndex);
+    m_pageIndexes[pageType] = pageIndex++;
+
     if (!icon.isEmpty())
       item->setIcon(QIcon{Q(":/icons/16x16/%1.png").arg(icon)});
 
@@ -153,26 +176,24 @@ PreferencesDialog::setupPageSelector() {
     return item;
   };
 
-  auto page      = 0;
-  auto pGui      = addItem(page++, nullptr, QY("GUI"),               "mkvtoolnix-gui");
-                   addItem(page++, pGui,    QY("Often used selections"));
-  auto pMerge    = addItem(page++, nullptr, QY("Multiplexer"),       "merge");
-                   addItem(page++, pMerge,  QY("Default values"));
-                   addItem(page++, pMerge,  QY("Output"));
-                   addItem(page++, pMerge,  QY("Enabling tracks"));
-                   addItem(page++, pMerge,  QY("Playlists"));
-                   addItem(page++, nullptr, QY("Header editor"),     "document-edit");
-                   addItem(page++, nullptr, QY("Chapter editor"),    "story-editor");
-  auto pJobs     = addItem(page++, nullptr, QY("Jobs & job queue"),  "view-task");
-                   addItem(page++, pJobs,   QY("Executing programs"));
+  auto pGui      = addItem(Page::Gui,                 nullptr, QY("GUI"),                   "mkvtoolnix-gui");
+                   addItem(Page::OftenUsedSelections, pGui,    QY("Often used selections"));
+  auto pMerge    = addItem(Page::Merge,               nullptr, QY("Multiplexer"),           "merge");
+                   addItem(Page::DefaultValues,       pMerge,  QY("Default values"));
+                   addItem(Page::Output,              pMerge,  QY("Output"));
+                   addItem(Page::EnablingTracks,      pMerge,  QY("Enabling tracks"));
+                   addItem(Page::Playlists,           pMerge,  QY("Playlists"));
+                   addItem(Page::HeaderEditor,        nullptr, QY("Header editor"),         "document-edit");
+                   addItem(Page::ChapterEditor,       nullptr, QY("Chapter editor"),        "story-editor");
+  auto pJobs     = addItem(Page::Jobs,                nullptr, QY("Jobs & job queue"),      "view-task");
+                   addItem(Page::RunPrograms,         pJobs,   QY("Executing actions"));
 
   for (auto row = 0, numRows = model->rowCount(); row < numRows; ++row)
     ui->pageSelector->setExpanded(model->index(row, 0), true);
 
   ui->pageSelector->setMinimumSize(ui->pageSelector->minimumSizeHint());
 
-  auto selection = QItemSelection{model->index(0, 0), model->index(0, 0)};
-  ui->pageSelector->selectionModel()->select(selection, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current);
+  showPage(pageToShow);
 
   connect(ui->pageSelector->selectionModel(), &QItemSelectionModel::currentChanged, this, &PreferencesDialog::pageSelectionChanged);
 }
@@ -222,6 +243,11 @@ PreferencesDialog::setupToolTips() {
                    Q("%1 %2")
                    .arg(QY("Blu-ray discs often contain a chapter entry very close to the end of the movie."))
                    .arg(QY("If enabled the last entry will be skipped when loading chapters from such playlists in the chapter editor if it is located within five seconds of the end of the movie.")));
+  Util::setToolTip(ui->cbCETextFileCharacterSet,
+                   Q("%1 %2 %3")
+                   .arg(QY("The chapter editor needs to know the character set a text chapter file uses in order to display all characters properly."))
+                   .arg(QY("By default it always asks the user which character set to use when opening a file for which it cannot be recognized automatically."))
+                   .arg(QY("If a character set is selected here, it will be used instead of asking the user.")));
 
   // Merge page
   Util::setToolTip(ui->cbMAutoSetFileTitle,
@@ -264,12 +290,13 @@ PreferencesDialog::setupToolTips() {
                      .arg(QYH("The horizontal layout with two fixed columns results in a wider window while the vertical tab widget layout results in a higher window.")));
 
   Util::setToolTip(ui->cbMClearMergeSettings,
-                   Q("<p>%1</p><ol><li>%2 %3</li><li>%4 %5</li></ol>")
+                   Q("<p>%1</p><ol><li>%2 %3</li><li>%4 %5</li><li>%6</li></ol>")
                    .arg(QYH("The GUI can help you start your next multiplex settings after having started a job or having added a one to the job queue."))
                    .arg(QYH("With \"create new settings\" a new set of multiplex settings will be added."))
                    .arg(QYH("The current multiplex settings will be closed."))
                    .arg(QYH("With \"remove source files\" all source files will be removed."))
-                   .arg(QYH("Most of the other settings on the output tab will be kept intact, though.")));
+                   .arg(QYH("Most of the other settings on the output tab will be kept intact, though."))
+                   .arg(QYH("With \"close current settings\" the current multiplex settings will be closed without opening new ones.")));
 
   Util::setToolTip(ui->cbMAddingAppendingFilesPolicy,
                    Q("%1 %2 %3")
@@ -511,7 +538,7 @@ PreferencesDialog::setupEnableMuxingTracksByLanguage() {
 
 void
 PreferencesDialog::setupMergeAddingAppendingFilesPolicy() {
-  ui->cbMAddingAppendingFilesPolicy->addItem(QY("Ask the user"),                                                  static_cast<int>(Util::Settings::MergeAddingAppendingFilesPolicy::Ask));
+  ui->cbMAddingAppendingFilesPolicy->addItem(QY("Always ask the user"),                                           static_cast<int>(Util::Settings::MergeAddingAppendingFilesPolicy::Ask));
   ui->cbMAddingAppendingFilesPolicy->addItem(QY("Add all files to the current multiplex settings"),               static_cast<int>(Util::Settings::MergeAddingAppendingFilesPolicy::Add));
   ui->cbMAddingAppendingFilesPolicy->addItem(QY("Create one new multiplex settings tab and add all files there"), static_cast<int>(Util::Settings::MergeAddingAppendingFilesPolicy::AddToNew));
   ui->cbMAddingAppendingFilesPolicy->addItem(QY("Create one new multiplex settings tab for each file"),           static_cast<int>(Util::Settings::MergeAddingAppendingFilesPolicy::AddEachToNew));
@@ -525,7 +552,7 @@ PreferencesDialog::setupMergeAddingAppendingFilesPolicy() {
 
 void
 PreferencesDialog::setupHeaderEditorDroppedFilesPolicy() {
-  ui->cbHEDroppedFilesPolicy->addItem(QY("Ask the user"),                                        static_cast<int>(Util::Settings::HeaderEditorDroppedFilesPolicy::Ask));
+  ui->cbHEDroppedFilesPolicy->addItem(QY("Always ask the user"),                                 static_cast<int>(Util::Settings::HeaderEditorDroppedFilesPolicy::Ask));
   ui->cbHEDroppedFilesPolicy->addItem(QY("Open all files as tabs in the header editor"),         static_cast<int>(Util::Settings::HeaderEditorDroppedFilesPolicy::Open));
   ui->cbHEDroppedFilesPolicy->addItem(QY("Add all files as new attachments to the current tab"), static_cast<int>(Util::Settings::HeaderEditorDroppedFilesPolicy::AddAttachments));
 
@@ -577,9 +604,9 @@ PreferencesDialog::setupJobsRunPrograms() {
     auto widget = new PrefsRunProgramWidget{ui->twJobsPrograms, *runProgramConfig};
     ui->twJobsPrograms->addTab(widget, {});
 
-    setTabTitleForRunProgramExecutable(ui->twJobsPrograms->count() - 1, runProgramConfig->m_commandLine.value(0));
+    setTabTitleForRunProgramWidget(ui->twJobsPrograms->count() - 1, runProgramConfig->name());
 
-    connect(widget, &PrefsRunProgramWidget::executableChanged, this, &PreferencesDialog::setSendersTabTitleForRunProgramExecutable);
+    connect(widget, &PrefsRunProgramWidget::titleChanged, this, &PreferencesDialog::setSendersTabTitleForRunProgramWidget);
   }
 
   if (!m_cfg.m_runProgramConfigurations.isEmpty())
@@ -620,6 +647,7 @@ PreferencesDialog::save() {
   m_cfg.m_removeOldJobsDays                  = ui->sbGuiRemoveOldJobsDays->value();
 
   m_cfg.m_chapterNameTemplate                = ui->leCENameTemplate->text();
+  m_cfg.m_ceTextFileCharacterSet             = ui->cbCETextFileCharacterSet->currentData().toString();
   m_cfg.m_defaultChapterLanguage             = ui->cbCEDefaultLanguage->currentData().toString();
   m_cfg.m_defaultChapterCountry              = ui->cbCEDefaultCountry->currentData().toString();
   m_cfg.m_dropLastChapterFromBlurayPlaylist  = ui->cbCEDropLastFromBlurayPlaylist->isChecked();
@@ -682,7 +710,7 @@ PreferencesDialog::save() {
     auto widget = static_cast<PrefsRunProgramWidget *>(ui->twJobsPrograms->widget(idx));
     auto cfg    = widget->config();
 
-    if (cfg->isValid())
+    if (!cfg->m_active || cfg->isValid())
       m_cfg.m_runProgramConfigurations << cfg;
   }
 
@@ -733,12 +761,12 @@ PreferencesDialog::editDefaultAdditionalCommandLineOptions() {
 void
 PreferencesDialog::addProgramToExecute() {
   auto programWidget = new PrefsRunProgramWidget{this, {}};
-  ui->twJobsPrograms->addTab(programWidget, QY("<No program selected yet>"));
+  ui->twJobsPrograms->addTab(programWidget, programWidget->config()->name());
   ui->twJobsPrograms->setCurrentIndex(ui->twJobsPrograms->count() - 1);
 
   ui->swJobsPrograms->setCurrentIndex(1);
 
-  connect(programWidget, &PrefsRunProgramWidget::executableChanged, this, &PreferencesDialog::setSendersTabTitleForRunProgramExecutable);
+  connect(programWidget, &PrefsRunProgramWidget::titleChanged, this, &PreferencesDialog::setSendersTabTitleForRunProgramWidget);
 }
 
 void
@@ -758,18 +786,20 @@ PreferencesDialog::removeProgramToExecute(int index) {
 }
 
 void
-PreferencesDialog::setSendersTabTitleForRunProgramExecutable(QString const &executable) {
-  setTabTitleForRunProgramExecutable(ui->twJobsPrograms->indexOf(qobject_cast<PrefsRunProgramWidget *>(sender())), executable);
+PreferencesDialog::setSendersTabTitleForRunProgramWidget() {
+  auto widget = qobject_cast<PrefsRunProgramWidget *>(sender());
+  auto title  = widget->config()->name();
+
+  setTabTitleForRunProgramWidget(ui->twJobsPrograms->indexOf(widget), title);
 }
 
 void
-PreferencesDialog::setTabTitleForRunProgramExecutable(int tabIdx,
-                                                      QString const &executable) {
+PreferencesDialog::setTabTitleForRunProgramWidget(int tabIdx,
+                                                  QString const &title) {
   if ((tabIdx < 0) || (tabIdx >= ui->twJobsPrograms->count()))
     return;
 
-  auto name = executable.isEmpty() ? QY("<No program selected yet>") : QFileInfo{executable}.fileName();
-  ui->twJobsPrograms->setTabText(tabIdx, name);
+  ui->twJobsPrograms->setTabText(tabIdx, title);
 }
 
 void
@@ -783,4 +813,46 @@ PreferencesDialog::adjustRemoveOldJobsControls() {
   ui->sbGuiRemoveOldJobsDays->setSuffix(QNY(" day", " days", ui->sbGuiRemoveOldJobsDays->value()));
 }
 
+void
+PreferencesDialog::showPage(Page page) {
+  auto pageIndex      = m_pageIndexes[page];
+  auto pageModelIndex = modelIndexForPage(pageIndex);
+
+  if (!pageModelIndex.isValid())
+    return;
+
+  m_ignoreNextCurrentChange = true;
+  auto selection            = QItemSelection{pageModelIndex, pageModelIndex};
+
+  ui->pageSelector->selectionModel()->select(selection, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Current);
+  ui->pages->setCurrentIndex(pageIndex);
+}
+
+void
+PreferencesDialog::accept() {
+  for (auto tabIdx = 0, numTabs = ui->twJobsPrograms->count(); tabIdx < numTabs; ++tabIdx) {
+    auto tab   = qobject_cast<PrefsRunProgramWidget *>(ui->twJobsPrograms->widget(tabIdx));
+    auto error = tab->validate();
+
+    if (error.isEmpty())
+      continue;
+
+    showPage(Page::RunPrograms);
+    ui->twJobsPrograms->setCurrentIndex(tabIdx);
+
+    Util::MessageBox::critical(this)
+      ->title(QY("Invalid settings"))
+      .text(Q("<p>%1 %2</p>"
+              "<p>%3</p>")
+            .arg(QY("This configuration is currently invalid.").toHtmlEscaped())
+            .arg(error.toHtmlEscaped())
+            .arg(QY("Either fix the error or remove the configuration before closing the preferences dialog.").toHtmlEscaped()))
+      .exec();
+
+    return;
+  }
+
+  QDialog::accept();
+}
+
 }}
