@@ -39,6 +39,9 @@ QString itemFlagsToString(Qt::ItemFlags const &flags);
 
 QString mapToTopLevelCountryCode(QString const &countryCode);
 
+QString replaceApplicationDirectoryWithMtxVariable(QString string);
+QString replaceMtxVariableWithApplicationDirectory(QString string);
+
 class DeferredRegularExpression {
 private:
   std::unique_ptr<QRegularExpression> m_re;
