@@ -42,6 +42,7 @@ mp3_packetizer_c::mp3_packetizer_c(generic_reader_c *p_reader,
 {
   set_track_type(track_audio);
   set_track_default_duration(m_packet_duration);
+  m_timestamp_calculator.set_allow_smaller_timestamps(true);
 }
 
 mp3_packetizer_c::~mp3_packetizer_c() {
@@ -179,7 +180,7 @@ mp3_packetizer_c::set_headers() {
 
 int
 mp3_packetizer_c::process(packet_cptr packet) {
-  m_timestamp_calculator.add_timecode(packet);
+  m_timestamp_calculator.add_timestamp(packet);
 
   unsigned char *mp3_packet;
   mp3_header_t mp3header;
@@ -187,7 +188,7 @@ mp3_packetizer_c::process(packet_cptr packet) {
   m_byte_buffer.add(packet->data->get_buffer(), packet->data->get_size());
 
   while ((mp3_packet = get_mp3_packet(&mp3header))) {
-    auto new_timecode = m_timestamp_calculator.get_next_timecode(m_samples_per_frame);
+    auto new_timecode = m_timestamp_calculator.get_next_timestamp(m_samples_per_frame);
     auto packet       = std::make_shared<packet_t>(memory_c::clone(mp3_packet, mp3header.framesize), new_timecode.to_ns(), m_packet_duration);
 
     packet->add_extensions(m_packet_extensions);
