@@ -11,7 +11,7 @@ TEST(MmIo, Slurp) {
   memory_cptr m;
 
   ASSERT_NO_THROW(m = mm_file_io_c::slurp("tests/unit/data/text/chunky_bacon.txt"));
-  EXPECT_EQ(*m, std::string{"Chunky Bacon\n"});
+  EXPECT_EQ(std::string{"Chunky Bacon\n"}, *m);
 
   ASSERT_THROW(mm_file_io_c::slurp("doesnotexist"), mtx::mm_io::exception);
 }
